/*****************************************************************************
SBCounterV4 is an improved counter.

The differences to the stock counter are:
* Count and Complete messages can be broadcasted.
* Supports until 2147483648 (=2^31, data type int) counts instead of 256 (=2^8, data type byte).
* The first <em>n</em> counter events can be skipped (useful for for triggered Creature-/ThingFactories)
* On each count an Event can be risen (property "CounterEvent").
* Counting can be selected through an Actor class.

14 Nov 2020, SeriousBarbie AT googlemail MAKEADOT com
*****************************************************************************/
class SBCounter extends Triggers;

#exec TEXTURE IMPORT NAME=SBCounterPic FILE=Textures\SBCounterPic.pcx MIPS=OFF FLAGS=2

var(Counter) bool bBroadcast; // If displaying count message, display it at all players?
var(Counter) bool bDebug; // write things to log file
var(Counter) bool bShowMessage; // Display count message?
var(Counter) name CounterEvent; // Called every time after the counter has changed
var(Counter) localized string CountMessage; // Human readable count message.
var(Counter) localized string CompleteMessage; // Completion message.
var(Counter) int SkipFirstCounterEvents; // How many CounterEvents should be omitted? Set to "1" for triggered Creature-/ThingFactories and *NumToCount* to ThingFactory.Capacity + 1
var(Counter) int NumToCount; // Number to count down from
var(Counter) class<actor> Prototype; // the template class

// internal variables needed for Reset function:
var int OriginalSkipFirstCounterEvents;
var int OriginalNum; // Number to count at startup time.


function BeginPlay() {
	Super.BeginPlay();
	// needed for Reset function:
	OriginalSkipFirstCounterEvents = SkipFirstCounterEvents;
	OriginalNum = NumToCount;
}



function CopyFromCounter(Counter C) {
/******************************************************************************
Copies the needed property values from a counter.
******************************************************************************/
	if (C == None)
		return;

	NumToCount = C.NumToCount;
	bShowMessage = C.bShowMessage;
	CountMessage = C.CountMessage;
	CompleteMessage = C.CompleteMessage;
}



function bool DoDisplayMessage(string MsgTemplate, pawn EventInstigator) {
/******************************************************************************
Should/can a message be displayed?
******************************************************************************/
	if ( ! bShowMessage || MsgTemplate == "")
		return false;
	// skip message for EventInstigator, if none EventInstigator is given:
	return bBroadcast || (EventInstigator != None);
}



function DisplayMessage(string msg, pawn EventInstigator) {
/******************************************************************************
Displays *msg* either on *EventInstigator* only or broadcasts it to all
players.
******************************************************************************/
	if (bBroadcast)
		BroadcastMessage(Msg, false);
	else if (EventInstigator != None)
		EventInstigator.ClientMessage(msg);
}



function string GetDisplayMessage(int CounterNum, string MsgTemplate) {
/******************************************************************************
Returns the message to be displayed. If *CounterNum* <= 0, *MsgTemplate* is
returned unchanged.
If *CounterNum* > 0, the funtion returns *MsgTemplate* with all "%i" in it
replaced by *CounterNum*.
******************************************************************************/
local string NumStr, s;
local int i;

	if (CounterNum <= 0)
		return MsgTemplate;

	if (MsgTemplate == "")
		return "";

	switch(CounterNum)
	{
		case 1:  NumStr = "one"; break;
		case 2:  NumStr = "two"; break;
		case 3:  NumStr = "three"; break;
		case 4:  NumStr ="four"; break;
		case 5:  NumStr = "five"; break;
		case 6:  NumStr = "six"; break;
		default: NumStr = string(CounterNum); break;
	}
	S = MsgTemplate;
	while(InStr(S, "%i") >= 0 )
	{
		i = InStr(S, "%i");
		S = Left(S, i) $ NumStr $ Mid(S, i + 2);
	}
	return s;
}



function logger(string FunctionName, string msg) {
	log(self $ "." $ FunctionName $ ":" @ msg);
}



function Reset() {
	SkipFirstCounterEvents = OriginalSkipFirstCounterEvents;
	NumToCount = OriginalNum;
}



function Trigger(actor Other, pawn EventInstigator) {

	if (bDebug) logger("Trigger", "entered with Other=" $ Other);
	if (NumToCount <= 0)
	{
		if (bDebug) logger("Trigger", "NumToCount <= 0, exiting");
		return;
	}

	// if prototype is set, count only if Other is from that factory prototype
	if (Prototype != None)
		if (Other == None || Other.class != Prototype)
		{
			if (bDebug) logger("Trigger", "Other.class != Prototype, exiting");
			return;
		}


	if (--NumToCount == 0)
	{
		// completed, now trigger all matching actors.
		if (DoDisplayMessage(CompleteMessage, EventInstigator))
			DisplayMessage(GetDisplayMessage(0, CompleteMessage), EventInstigator);
		TriggerOthers(Other, EventInstigator, Event);
	}
	else
	{
		if (SkipFirstCounterEvents > 0)
			SkipFirstCounterEvents--;
		else
		{
			if (DoDisplayMessage(CountMessage, EventInstigator))
				DisplayMessage(GetDisplayMessage(NumToCount, CountMessage), EventInstigator);
			TriggerOthers(Other, EventInstigator, CounterEvent);
		}
	}
}



function TriggerOthers(actor Other, pawn EventInstigator, name TriggerEvent) {
/******************************************************************************
Just triggers all other Actors with Tag==TriggerEvent
******************************************************************************/
local actor A;

	if (bDebug) logger("TriggerOthers", "entered with Event=" $ TriggerEvent);
	if (TriggerEvent != '')
		foreach AllActors(class 'Actor', A, TriggerEvent)
			A.Trigger(Other, EventInstigator);
}

auto state Startup{

begin:
	if (bDebug)
	{
		logger("PostBeginPlay", "bBroadcast=" $ bBroadcast);
		logger("PostBeginPlay", "bShowMessage=" $ bShowMessage);
		logger("PostBeginPlay", "CounterEvent=" $ CounterEvent);
		logger("PostBeginPlay", "CountMessage=" $ CountMessage);
		logger("PostBeginPlay", "CompleteMessage=" $ CompleteMessage);
		logger("PostBeginPlay", "SkipFirstCounterEvents=" $ SkipFirstCounterEvents);
		logger("PostBeginPlay", "NumToCount=" $ NumToCount);
		logger("PostBeginPlay", "Prototype=" $ Prototype);
	}
}



defaultproperties {
	bBroadcast=false
	bDebug=false
	bShowMessage=false
	CompleteMessage="Completed!"
	CountMessage="Only %i more to go..."
	NumToCount=2
	Texture=SBCounterPic
}
